@@ -32,7 +32,10 @@
 #include <string.h>
 #endif
 
+#include <libxfce4ui/libxfce4ui.h>
+
 #include <thunar/thunar-dialogs.h>
+#include <thunar/thunar-gtk-extensions.h>
 #include <thunar/thunar-icon-factory.h>
 #include <thunar/thunar-io-jobs.h>
 #include <thunar/thunar-job.h>
@@ -45,9 +48,134 @@
 
 static void          thunar_dialogs_select_filename      (GtkWidget  *entry,
                                                           ThunarFile *file);
-static gboolean      thunar_dialogs_entry_undo           (GtkWidget  *widget,
-                                                          GdkEvent   *event,
-                                                          ThunarFile *file);
+
+
+
+/**
+ * thunar_dialogs_show_create:
+ * @parent       : a #GdkScreen, a #GtkWidget or %NULL.
+ * @content_type : the content type of the file or folder to create.
+ * @filename     : the suggested filename or %NULL.
+ * @title        : the dialog title.
+ *
+ * Displays a Thunar create dialog  with the specified
+ * parameters that asks the user to enter a name for a new file or
+ * folder.
+ *
+ * The caller is responsible to free the returned filename using
+ * g_free() when no longer needed.
+ *
+ * Return value: the filename entered by the user or %NULL if the user
+ *               cancelled the dialog.
+ **/
+gchar*
+thunar_dialogs_show_create (gpointer     parent,
+                            const gchar *content_type,
+                            const gchar *filename,
+                            const gchar *title)
+{
+  GtkWidget         *dialog;
+  GtkWindow         *window;
+  GdkScreen         *screen;
+  GError            *error = NULL;
+  gchar             *name = NULL;
+  GtkWidget         *label;
+  GtkWidget         *grid;
+  GtkWidget         *image;
+  XfceFilenameInput *filename_input;
+  GIcon             *icon = NULL;
+
+  _thunar_return_val_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent), NULL);
+
+  /* parse the parent window and screen */
+  screen = thunar_util_parse_parent (parent, &window);
+
+  /* create a new dialog window */
+  dialog = gtk_dialog_new_with_buttons (title,
+                                        window,
+                                        GTK_DIALOG_MODAL
+                                        | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                        _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                        _("C_reate"), GTK_RESPONSE_OK,
+                                        NULL);
+
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, FALSE);
+  gtk_window_set_default_size (GTK_WINDOW (dialog), 300, -1);
+
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 3);
+  gtk_container_set_border_width (GTK_CONTAINER (grid), 6);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), grid, TRUE, TRUE, 0);
+  gtk_widget_show (grid);
+
+  /* try to load the icon */
+  if (G_LIKELY (content_type != NULL))
+    icon = g_content_type_get_icon (content_type);
+
+  /* setup the image */
+  if (G_LIKELY (icon != NULL))
+    {
+      image = g_object_new (GTK_TYPE_IMAGE, "xpad", 6, "ypad", 6, NULL);
+      gtk_image_set_from_gicon (GTK_IMAGE (image), icon, GTK_ICON_SIZE_DIALOG);
+      gtk_grid_attach (GTK_GRID (grid), image, 0, 0, 1, 2);
+      g_object_unref (icon);
+      gtk_widget_show (image);
+    }
+
+  label = g_object_new (GTK_TYPE_LABEL, "label", _("Enter the name:"), "xalign", 0.0f, "hexpand", TRUE, NULL);
+  gtk_grid_attach (GTK_GRID (grid), label, 1, 0, 1, 1);
+  gtk_widget_show (label);
+
+  /* set up the widget for entering the filename */
+  filename_input = g_object_new (XFCE_TYPE_FILENAME_INPUT, "original-filename", filename, NULL);
+  gtk_widget_set_hexpand (GTK_WIDGET (filename_input), TRUE);
+  gtk_widget_set_valign (GTK_WIDGET (filename_input), GTK_ALIGN_CENTER);
+
+  /* connect to signals so that the sensitivity of the Create button is updated according to whether there
+   * is a valid file name entered */
+  g_signal_connect_swapped (filename_input, "text-invalid", G_CALLBACK (xfce_filename_input_desensitise_widget),
+                            gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK));
+  g_signal_connect_swapped (filename_input, "text-valid", G_CALLBACK (xfce_filename_input_sensitise_widget),
+                            gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK));
+
+  gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (filename_input), 1, 1, 1, 1);
+  thunar_gtk_label_set_a11y_relation (GTK_LABEL (label),
+                                      GTK_WIDGET (xfce_filename_input_get_entry (filename_input)));
+  gtk_widget_show_all ( GTK_WIDGET (filename_input));
+
+  /* ensure that the sensitivity of the Create button is set correctly */
+  xfce_filename_input_check (filename_input);
+
+  if (screen != NULL)
+    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+
+  if (window != NULL)
+    gtk_window_set_transient_for (GTK_WINDOW (dialog), window);
+
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
+    {
+      /* determine the chosen filename */
+      filename = xfce_filename_input_get_text (filename_input);
+
+      /* convert the UTF-8 filename to the local file system encoding */
+      name = g_filename_from_utf8 (filename, -1, NULL, NULL, &error);
+      if (G_UNLIKELY (name == NULL))
+        {
+          /* display an error message */
+          thunar_dialogs_show_error (dialog, error, _("Cannot convert filename \"%s\" to the local encoding"), filename);
+
+          /* release the error */
+          g_error_free (error);
+        }
+    }
+
+  /* destroy the dialog */
+  gtk_widget_destroy (dialog);
+
+  return name;
+}
 
 
 
@@ -73,7 +201,6 @@ thunar_dialogs_show_rename_file (gpointer    parent,
   const gchar       *text;
   ThunarJob         *job = NULL;
   GtkWidget         *dialog;
-  GtkWidget         *entry;
   GtkWidget         *label;
   GtkWidget         *image;
   GtkWidget         *grid;
@@ -86,6 +213,7 @@ thunar_dialogs_show_rename_file (gpointer    parent,
   gint               layout_width;
   gint               layout_offset;
   gint               parent_width = 500;
+  XfceFilenameInput *filename_input;
 
   _thunar_return_val_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WINDOW (parent), FALSE);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
@@ -139,27 +267,33 @@ thunar_dialogs_show_rename_file (gpointer    parent,
   gtk_grid_attach (GTK_GRID (grid), label, 1, 0, 1, 1);
   gtk_widget_show (label);
 
-  entry = gtk_entry_new ();
-  gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
-  gtk_widget_set_hexpand (entry, TRUE);
-  gtk_widget_set_valign (entry, GTK_ALIGN_CENTER);
-  gtk_grid_attach (GTK_GRID (grid), entry, 1, 1, 1, 1);
-  gtk_widget_show (entry);
+  /* set up the widget for entering the filename */
+  filename_input = g_object_new (XFCE_TYPE_FILENAME_INPUT, "original-filename", filename, NULL);
+  gtk_widget_set_hexpand (GTK_WIDGET (filename_input), TRUE);
+  gtk_widget_set_valign (GTK_WIDGET (filename_input), GTK_ALIGN_CENTER);
 
-  /* setup the old filename */
-  gtk_entry_set_text (GTK_ENTRY (entry), filename);
+  /* connect to signals so that the sensitivity of the Create button is updated according to whether there
+   * is a valid file name entered */
+  g_signal_connect_swapped (filename_input, "text-invalid", G_CALLBACK (xfce_filename_input_desensitise_widget),
+                            gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK));
+  g_signal_connect_swapped (filename_input, "text-valid", G_CALLBACK (xfce_filename_input_sensitise_widget),
+                            gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK));
 
-  /* allow reverting the filename with ctrl + z */
-  g_signal_connect (entry, "key-press-event",
-                    G_CALLBACK (thunar_dialogs_entry_undo), file);
+  gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (filename_input), 1, 1, 1, 1);
+  thunar_gtk_label_set_a11y_relation (GTK_LABEL (label),
+                                      GTK_WIDGET (xfce_filename_input_get_entry (filename_input)));
+  gtk_widget_show_all ( GTK_WIDGET (filename_input));
+
+  /* ensure that the sensitivity of the Create button is set correctly */
+  xfce_filename_input_check (filename_input);
 
   /* select the filename without the extension */
-  thunar_dialogs_select_filename (entry, file);
+  thunar_dialogs_select_filename (GTK_WIDGET (xfce_filename_input_get_entry (filename_input)), file);
 
   /* get the size the entry requires to render the full text */
-  layout = gtk_entry_get_layout (GTK_ENTRY (entry));
+  layout = gtk_entry_get_layout (xfce_filename_input_get_entry (filename_input));
   pango_layout_get_pixel_size (layout, &layout_width, NULL);
-  gtk_entry_get_layout_offsets (GTK_ENTRY (entry), &layout_offset, NULL);
+  gtk_entry_get_layout_offsets (xfce_filename_input_get_entry (filename_input), &layout_offset, NULL);
   layout_width += (layout_offset * 2) + (12 * 4) + 48; /* 12px free space in entry */
 
   /* parent window width */
@@ -185,7 +319,7 @@ thunar_dialogs_show_rename_file (gpointer    parent,
       gtk_widget_hide (dialog);
 
       /* determine the new filename */
-      text = gtk_entry_get_text (GTK_ENTRY (entry));
+      text = xfce_filename_input_get_text (filename_input);
 
       /* check if we have a new name here */
       if (G_LIKELY (!exo_str_is_equal (filename, text)))
@@ -257,9 +391,6 @@ thunar_dialogs_show_about (GtkWindow   *parent,
   comments = g_strdup_vprintf (format, args);
   va_end (args);
 
-  /* try to load the about logo */
-  logo = gdk_pixbuf_new_from_file (DATADIR "/pixmaps/Thunar/Thunar-about-logo.png", NULL);
-
   /* open the about dialog */
   gtk_show_about_dialog (parent,
                          "artists", artists,
@@ -268,12 +399,12 @@ thunar_dialogs_show_about (GtkWindow   *parent,
                          "copyright", "Copyright \302\251 2004-2011 Benedikt Meurer\n"
                                       "Copyright \302\251 2009-2011 Jannis Pohlmann\n"
                                       "Copyright \302\251 2009-2012 Nick Schermer\n"
-                                      "Copyright \302\251 2017-2019 Alexander Schwinn\n"
-                                      "Copyright \302\251 2017-2019 Andre Miranda",
+                                      "Copyright \302\251 2017-2020 Alexander Schwinn\n"
+                                      "Copyright \302\251 2017-2020 Andre Miranda",
                          "destroy-with-parent", TRUE,
                          "documenters", documenters,
                          "license", XFCE_LICENSE_GPL,
-                         "logo", logo,
+                         "logo-icon-name", "org.xfce.thunar",
                          "program-name", title,
                          "translator-credits", _("translator-credits"),
                          "version", PACKAGE_VERSION,
@@ -441,7 +572,7 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message), "%s", secondary->str);
 
   /* add the buttons based on the possible choices */
-  for (n = 6; n >= 0; --n)
+  for (n = THUNAR_JOB_RESPONSE_MAX_INT; n >= 0; --n)
     {
       /* check if the response is set */
       response = choices & (1 << n);
@@ -458,6 +589,30 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
           mnemonic = _("Yes to _all");
           break;
 
+        case THUNAR_JOB_RESPONSE_REPLACE:
+          mnemonic = _("_Replace");
+          break;
+
+        case THUNAR_JOB_RESPONSE_REPLACE_ALL:
+          mnemonic = _("Replace _All");
+          break;
+
+        case THUNAR_JOB_RESPONSE_SKIP:
+          mnemonic = _("_Skip");
+          break;
+
+        case THUNAR_JOB_RESPONSE_SKIP_ALL:
+          mnemonic = _("S_kip All");
+          break;
+
+        case THUNAR_JOB_RESPONSE_RENAME:
+          mnemonic = _("Re_name");
+          break;
+
+        case THUNAR_JOB_RESPONSE_RENAME_ALL:
+          mnemonic = _("Rena_me All");
+          break;
+
         case THUNAR_JOB_RESPONSE_NO:
           mnemonic = _("_No");
           break;
@@ -563,6 +718,8 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   GtkWidget         *skip_button;
   GtkWidget         *replaceall_button;
   GtkWidget         *replace_button;
+  GtkWidget         *renameall_button;
+  GtkWidget         *rename_button;
   GdkPixbuf         *icon;
   gchar             *date_custom_style;
   gchar             *date_string;
@@ -588,7 +745,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
   gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
   gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), THUNAR_JOB_RESPONSE_YES);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), THUNAR_JOB_RESPONSE_REPLACE);
   content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
   /* determine the icon factory to use */
@@ -610,24 +767,32 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   skip_button       = gtk_button_new_with_mnemonic (_("_Skip"));
   replaceall_button = gtk_button_new_with_mnemonic (_("Replace _All"));
   replace_button    = gtk_button_new_with_mnemonic (_("_Replace"));
+  renameall_button  = gtk_button_new_with_mnemonic (_("Rena_me All"));
+  rename_button     = gtk_button_new_with_mnemonic (_("Re_name"));
 
   g_signal_connect (cancel_button,      "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
   g_signal_connect (skipall_button,     "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
   g_signal_connect (skip_button,        "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
   g_signal_connect (replaceall_button,  "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
   g_signal_connect (replace_button,     "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
+  g_signal_connect (renameall_button,   "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
+  g_signal_connect (rename_button,      "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
 
   g_object_set_data (G_OBJECT (cancel_button),     "response-id", GINT_TO_POINTER (GTK_RESPONSE_CANCEL));
-  g_object_set_data (G_OBJECT (skipall_button),    "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_NO_ALL));
-  g_object_set_data (G_OBJECT (skip_button),       "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_NO));
-  g_object_set_data (G_OBJECT (replaceall_button), "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_YES_ALL));
-  g_object_set_data (G_OBJECT (replace_button),    "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_YES));
+  g_object_set_data (G_OBJECT (skipall_button),    "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_SKIP_ALL));
+  g_object_set_data (G_OBJECT (skip_button),       "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_SKIP));
+  g_object_set_data (G_OBJECT (replaceall_button), "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_REPLACE_ALL));
+  g_object_set_data (G_OBJECT (replace_button),    "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_REPLACE));
+  g_object_set_data (G_OBJECT (renameall_button),  "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_RENAME_ALL));
+  g_object_set_data (G_OBJECT (rename_button),     "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_RENAME));
 
   gtk_container_add (GTK_CONTAINER (button_box), cancel_button);
   gtk_container_add (GTK_CONTAINER (button_box), skipall_button);
   gtk_container_add (GTK_CONTAINER (button_box), skip_button);
   gtk_container_add (GTK_CONTAINER (button_box), replaceall_button);
   gtk_container_add (GTK_CONTAINER (button_box), replace_button);
+  gtk_container_add (GTK_CONTAINER (button_box), renameall_button);
+  gtk_container_add (GTK_CONTAINER (button_box), rename_button);
   gtk_container_add (GTK_CONTAINER (content_area), button_box);
   gtk_widget_set_halign (button_box, GTK_ALIGN_CENTER);
   gtk_box_set_spacing (GTK_BOX (button_box), 5);
@@ -670,11 +835,14 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   g_free (text);
 
   if (thunar_file_is_symlink (dst_file))
-    text = g_strdup_printf (Q_("ReplaceDialogPart1|Do you want to replace the link"));
+    /* TRANSLATORS: First part of replace dialog sentence */
+    text = g_strdup_printf (_("Do you want to replace the link"));
   else if (thunar_file_is_directory (dst_file))
-    text = g_strdup_printf (Q_("ReplaceDialogPart1|Do you want to replace the existing folder"));
+    /* TRANSLATORS: First part of replace dialog sentence */
+    text = g_strdup_printf (_("Do you want to replace the existing folder"));
   else
-    text = g_strdup_printf (Q_("ReplaceDialogPart1|Do you want to replace the existing file"));
+    /* TRANSLATORS: First part of replace dialog sentence */
+    text = g_strdup_printf (_("Do you want to replace the existing file"));
 
   label = gtk_label_new (text);
   gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
@@ -706,11 +874,14 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   g_free (text);
 
   if (thunar_file_is_symlink (src_file))
-    text = g_strdup_printf (Q_("ReplaceDialogPart2|with the following link?"));
+    /* TRANSLATORS: Second part of replace dialog sentence */
+    text = g_strdup_printf (_("with the following link?"));
   else if (thunar_file_is_directory (src_file))
-    text = g_strdup_printf (Q_("ReplaceDialogPart2|with the following folder?"));
+    /* TRANSLATORS: Second part of replace dialog sentence */
+    text = g_strdup_printf (_("with the following folder?"));
   else
-    text = g_strdup_printf (Q_("ReplaceDialogPart2|with the following file?"));
+    /* TRANSLATORS: Second part of replace dialog sentence */
+    text = g_strdup_printf (_("with the following file?"));
 
   label = gtk_label_new (text);
   gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
@@ -953,28 +1124,3 @@ static void thunar_dialogs_select_filename (GtkWidget  *entry,
   if (G_LIKELY (offset > 0))
     gtk_editable_select_region (GTK_EDITABLE (entry), 0, offset);
 }
-
-
-
-static gboolean thunar_dialogs_entry_undo (GtkWidget  *widget,
-                                           GdkEvent   *event,
-                                           ThunarFile *file)
-{
-  guint           keyval;
-  GdkModifierType state;
-
-  if (G_UNLIKELY (!gdk_event_get_keyval (event, &keyval) ||
-                  !gdk_event_get_state (event, &state)))
-    return GDK_EVENT_PROPAGATE;
-
-  if ((state & GDK_CONTROL_MASK) != 0 && keyval == GDK_KEY_z)
-    {
-      gtk_entry_set_text (GTK_ENTRY (widget),
-                          thunar_file_get_display_name (file));
-      thunar_dialogs_select_filename (widget, file);
-
-      return GDK_EVENT_STOP;
-    }
-
-  return GDK_EVENT_PROPAGATE;
-}
