@@ -229,26 +229,39 @@ ThunarThumbnailSize thunar_zoom_level_to_thumbnail_size   (ThunarZoomLevel zoom_
 
 /**
  * ThunarJobResponse:
- * @THUNAR_JOB_RESPONSE_YES     :
- * @THUNAR_JOB_RESPONSE_YES_ALL :
- * @THUNAR_JOB_RESPONSE_NO      :
- * @THUNAR_JOB_RESPONSE_NO_ALL  :
- * @THUNAR_JOB_RESPONSE_CANCEL  :
- * @THUNAR_JOB_RESPONSE_RETRY   :
- * @THUNAR_JOB_RESPONSE_FORCE   :
+ * @THUNAR_JOB_RESPONSE_YES         :
+ * @THUNAR_JOB_RESPONSE_YES_ALL     :
+ * @THUNAR_JOB_RESPONSE_NO          :
+ * @THUNAR_JOB_RESPONSE_NO_ALL      :
+ * @THUNAR_JOB_RESPONSE_CANCEL      :
+ * @THUNAR_JOB_RESPONSE_RETRY       :
+ * @THUNAR_JOB_RESPONSE_FORCE       :
+ * @THUNAR_JOB_RESPONSE_REPLACE     :
+ * @THUNAR_JOB_RESPONSE_REPLACE_ALL :
+ * @THUNAR_JOB_RESPONSE_SKIP        :
+ * @THUNAR_JOB_RESPONSE_SKIP_ALL    :
+ * @THUNAR_JOB_RESPONSE_RENAME      :
+ * @THUNAR_JOB_RESPONSE_RENAME_ALL  :
  *
  * Possible responses for the ThunarJob::ask signal.
  **/
 typedef enum /*< flags >*/
 {
-  THUNAR_JOB_RESPONSE_YES     = 1 << 0,
-  THUNAR_JOB_RESPONSE_YES_ALL = 1 << 1,
-  THUNAR_JOB_RESPONSE_NO      = 1 << 2,
-  THUNAR_JOB_RESPONSE_CANCEL  = 1 << 3,
-  THUNAR_JOB_RESPONSE_NO_ALL  = 1 << 4,
-  THUNAR_JOB_RESPONSE_RETRY   = 1 << 5,
-  THUNAR_JOB_RESPONSE_FORCE   = 1 << 6,
+  THUNAR_JOB_RESPONSE_YES         = 1 << 0,
+  THUNAR_JOB_RESPONSE_YES_ALL     = 1 << 1,
+  THUNAR_JOB_RESPONSE_NO          = 1 << 2,
+  THUNAR_JOB_RESPONSE_CANCEL      = 1 << 3,
+  THUNAR_JOB_RESPONSE_NO_ALL      = 1 << 4,
+  THUNAR_JOB_RESPONSE_RETRY       = 1 << 5,
+  THUNAR_JOB_RESPONSE_FORCE       = 1 << 6,
+  THUNAR_JOB_RESPONSE_REPLACE     = 1 << 7,
+  THUNAR_JOB_RESPONSE_REPLACE_ALL = 1 << 8,
+  THUNAR_JOB_RESPONSE_SKIP        = 1 << 9,
+  THUNAR_JOB_RESPONSE_SKIP_ALL    = 1 << 10,
+  THUNAR_JOB_RESPONSE_RENAME      = 1 << 11,
+  THUNAR_JOB_RESPONSE_RENAME_ALL  = 1 << 12,
 } ThunarJobResponse;
+#define THUNAR_JOB_RESPONSE_MAX_INT 12
 
 GType thunar_job_response_get_type (void) G_GNUC_CONST;
 
